import { isUndefined, reduce } from "lodash";
import stringify from "./stringify";

const defaultFeatures = require
  .context('./features', false, /\.js$/).keys()
  .reduce((memo, key) => ({
    ...memo,
    [key.replace(/\.\//,'').replace(/\.js$/,'')]: features(key),
  }), {})

const defaultReducers = require
  .context('./reducers', false, /\.js$/).keys()
  .reduce((memo, key) => ({
    ...memo,
    [key.replace(/\.\//,'').replace(/\.js$/,'')]: reducers(key),
  }), {})

export default function api(options = {}) {
  let { history = [], features = {}, reducers = {} } = options

  features = { ...defaultFeatures, ...features };
  reducers = { ...defaultReducers, ...reducers };

  const getState = () => {
    const initialState = reduce(features, (acc, feature, name) => {
      const defaultValue = feature();

      if (isUndefined(defaultValue)) {
        return acc;
      }

      return {
        ...acc,
        [name]: defaultValue,
      };
    }, {});

    const state = history.reduce((acc, { name, args }) => {
      const featureState = acc[name];
      const feature = features[name];

      return {
        ...acc,
        [name]: feature(featureState, ...args),
      };
    }, initialState);

    Object.defineProperty(state, "toString", {
      value: () => stringify(state),
    });

    return state;
  };

  const getConfig = (tap) => {
    const state = getState();
    let config = reduce(reducers, (config, reducer, name) => {
      const reduced = reducer(state);

      if (isUndefined(reduced)) {
        return config;
      }

      return {
        ...config,
        [name]: reducer(state),
      };
    }, {});

    if (tap) {
      config = tap(config)
    }

    Object.defineProperty(config, "toString", {
      value: () => stringify(config),
    });

    return config;
  };

  const when = (env, configure) => {
    const envs = Array.isArray(env) ? env : [env];
    const { NODE_ENV = "development" } = process.env;

    if (env === true || envs.indexOf(NODE_ENV) !== -1) {
      return configure(api({features,reducers,history}));
    } else if (typeof env === 'function' && env(getState(), getConfig)) {
      return configure({features, reducers, history})
    }

    return api({features, reducers, history});
  };

  return reduce(features, (acc, feature, name) => ({
    ...acc,
    [name]: (...args) => api(
      features,
      reducers,
      history.concat({ args, name }),
    ),
  }), {
    getConfig,
    getState,
    history,
    when,
  });
}
