jest.dontMock('../CCExpiryInput');
jest.dontMock('../../utils');

import React from 'react';
import ReactDOM from 'react-dom';
import TestUtils from 'react-addons-test-utils';

const CCExpiryInput = require('../CCExpiryInput').default;
const callback = jest.genMockFn();

describe('CCExpiryInput lifecycle tests', () => {
  let component;
  let input;

  beforeEach(() => {
    component = TestUtils.renderIntoDocument(<CCExpiryInput onStateUpdate={callback}/>);
    input = component.refs.ccExpiryInput;
  });

  it('renders properly on load', () => {
    // Should exist
    expect(input).toBeTruthy();
    // Should be empty
    expect(input.textContent).toBe('');
    // Should have an empty state
    expect(component.state.value).toBe('');
  });

  it('handles onKeyDown properly', () => {
    // Set up a mock to make sure we're calling the method
    component.formatBackExpiry = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.keyDown(input, {keyCode: 65});

    // Check it
    expect(component.formatBackExpiry).toBeCalled();
  });

  it('handles onKeyPress properly', () => {
    // Set up some mocks to make sure we're calling the methods
    component.restrictExpiry = jest.genMockFn();
    component.formatExpiry = jest.genMockFn();
    component.formatForwardSlashAndSpace = jest.genMockFn();
    component.formatForwardExpiry = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.keyPress(input, {keyCode: 65});

    // Check it
    expect(component.restrictExpiry).toBeCalled();
    expect(component.formatExpiry).toBeCalled();
    expect(component.formatForwardSlashAndSpace).toBeCalled();
    expect(component.formatForwardExpiry).toBeCalled();
  });

  it('handles onInput properly', () => {
    // Set up some mocks to make sure we're calling the methods
    component.reformatExpiry = jest.genMockFn();

    // Trigger
    TestUtils.Simulate.input(input, {keyCode: 65});

    // Check it
    expect(component.reformatExpiry).toBeCalled();
  });

  it('calls props.onStateUpdate to change outer state', () => {
    // Set a new value
    input.value = '04 / 2015';

    // Change it
    TestUtils.Simulate.input(input);

    // Check it
    expect(callback).toBeCalled();
  });
});

describe('CCExpiryInput formatter tests', () => {
  let component;

  beforeEach(() => {
    component = TestUtils.renderIntoDocument(<CCExpiryInput onStateUpdate={callback}/>);
  });

  it('ensures getFormattedExpiry bails on non-digits', () => {
    expect(component.getFormattedExpiry('a')).toBe('');
  });

  it('ensures getFormattedExpiry formats exp_month properly', () => {
    expect(component.getFormattedExpiry('5')).toBe('05 / ');
    expect(component.getFormattedExpiry('05')).toBe('05 / ');
    expect(component.getFormattedExpiry('05/')).toBe('05 / ');
    expect(component.getFormattedExpiry('0')).toBe('0');
    expect(component.getFormattedExpiry('1')).toBe('1');
  });

  it('ensures getFormattedExpiry formats exp_year properly', () => {
    expect(component.getFormattedExpiry('052015')).toBe('05 / 2015');
    expect(component.getFormattedExpiry('0515')).toBe('05 / 15');
    expect(component.getFormattedExpiry('05 / 2015555')).toBe('05 / 2015');
  });

  it('ensures getFormattedExpiry formats the separator properly', () => {
    expect(component.getFormattedExpiry('5 /')).toBe('5');
  });

  it('ensures formatExpiry formats only numeric events', () => {
    // Set up a dummy event
    const event = {
      keyCode: 65,
      which: 65,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };

    // Check it
    const expiry = component.formatExpiry(event);
    expect(expiry).toBeUndefined();
  });

  it('ensures formatExpiry formats single digit entries', () => {
    // Set up a dummy event
    const event = {
      keyCode: 52,
      which: 52,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };

    // Check it
    component.formatExpiry(event);
    expect(component.state.value).toBe('04 / ');
  });

  it('ensures formatExpiry formats multi-digit month entries', () => {
    // Setup a dummy event
    const event = {
      keyCode: 54,
      which: 54,
      target: {value: '0'},
      preventDefault: jest.genMockFn()
    };

    // Check it
    component.formatExpiry(event);
    expect(component.state.value).toBe('06 / ');
  });

  it('ensures formatExpiry formats multi-digit year/month split', () => {
    // Setup a dummy event
    const event = {
      keyCode: 51,
      which: 51,
      target: {value: '1'},
      preventDefault: jest.genMockFn()
    };

    // Check it
    component.formatExpiry(event);
    expect(component.state.value).toBe('01 / 3');
  });

  it('ensures formatForwardSlashAndSpace works with slash', () => {
    // Setup a dummy '/' event
    const event = {
      keyCode: 47,
      which: 47,
      target: {value: '3'},
      preventDefault: jest.genMockFn()
    };
    component.formatForwardSlashAndSpace(event);
    expect(component.state.value).toBe('03 / ');
  });

  it('ensures formatForwardSlashAndSpace works with space', () => {
    // Set up a dummy ' ' event
    const event = {
      keyCode: 32,
      which: 32,
      target: {value: '1'},
      preventDefault: jest.genMockFn()
    };
    component.formatForwardSlashAndSpace(event);
    expect(component.state.value).toBe('01 / ');
  });

  it('ensures formatForwardSlashAndSpace only handles space and slash', () => {
    // Set up a dummy '4' event
    const event = {
      keyCode: 52,
      which: 52,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };
    const result = component.formatForwardSlashAndSpace(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatForwardExpiry formats only numerics', () => {
    // Dummy 'a' event
    const event = {
      keyCode: 65,
      which: 65,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };
    const result = component.formatForwardExpiry(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatForwardExpiry formats numerics properly', () => {
    // Dummy input event
    const event = {
      keyCode: 51,
      which: 51,
      target: {value: '04'},
      preventDefault: jest.genMockFn()
    };
    const result = component.formatForwardExpiry(event);
    expect(component.state.value).toBe('04 / ');
  });

  it('ensures formatBackExpiry formats only when backspacing', () => {
    // Dummy input event
    const event = {
      keyCode: 51,
      which: 51,
      target: {value: '04'},
      preventDefault: jest.genMockFn()
    };
    const result = component.formatBackExpiry(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatBackExpiry wont format partially selected text', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {
        value: '04',
        selectionStart: 2
      },
      preventDefault: jest.genMockFn()
    };
    const result = component.formatBackExpiry(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatBackExpiry wont format mid-input selections', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {
        value: '04',
        selectionStart: 1
      },
      preventDefault: jest.genMockFn()
    };
    const result = component.formatBackExpiry(event);
    expect(result).toBeUndefined();
  });

  it('ensures formatBackExpiry removes the trailing space/last digit', () => {
    // Dummy backspace event
    const event = {
      keyCode: 8,
      which: 8,
      target: {value: '04 / ', selectionStart: null},
      preventDefault: jest.genMockFn()
    };
    component.formatBackExpiry(event);
    expect(component.state.value).toBe('0');
  });

  it('ensures formatExpiry overrides state', () => {
    // Dummy event
    const event = {
      target: {value: '04/2015'}
    };
    component.reformatExpiry(event);
    expect(component.state.value).toBe('04 / 2015');
  });

  it('ensures cardExpiryVal returns object properly, 4-digit year', () => {
    // Force an initial state
    const event = {
      target: {value: '04/2015'}
    };
    component.reformatExpiry(event);
    const result = component.cardExpiryVal();

    // Check it
    expect(result.month).toBe(4);
    expect(result.year).toBe(2015);
  });

  it('ensures cardExpiryVal returns object properly, 2-digit year', () => {
    // Force an initial state
    const event = {
      target: {value: '04/15'}
    };
    component.reformatExpiry(event);
    const result = component.cardExpiryVal();

    // Check it
    expect(result.month).toBe(4);
    expect(result.year).toBe(2015);
  });
});

describe('CCExpiryInput validation tests', () => {
  let component;

  beforeEach(() => {
    component = TestUtils.renderIntoDocument(<CCExpiryInput onStateUpdate={callback}/>);
  });

  it('ensures validate fails without an initial state', () => {
    expect(component.validate()).toBeFalsy();
  });

  it('ensures validate fails with bad months', () => {
    // Force initial state
    component.reformatExpiry({
      target: {value: '15 / 2015'}
    });

    expect(component.validate()).toBeFalsy();
  });

  it('ensures validate passes', () => {
    // Force initial state
    component.reformatExpiry({
      target: {value: '10 / 2020'}
    });

    expect(component.validate()).toBeTruthy();
  });

  it('ensures validate fails with old years', () => {
    // Force initial state
    component.reformatExpiry({
      target: {value: '10 / 1970'}
    });

    expect(component.validate()).toBeFalsy();
  });

  it('ensures restrictExpiry allows digits', () => {
    // Dummy event
    const event = {
      keyCode: 53,
      which: 53,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };
    // We dont need to restrict this
    expect(component.restrictExpiry(event)).toBeFalsy();
  });

  it('ensures restrictExpiry denys letters', () => {
    // Dummy event
    const event = {
      keyCode: 65,
      which: 65,
      target: {value: ''},
      preventDefault: jest.genMockFn()
    };
    // We need to restrict this
    expect(component.restrictExpiry(event)).toBeTruthy();
  });

  it('ensures restrictExpiry restricts selected text', () => {
    // Dummy event
    const event = {
      keyCode: 53,
      which: 53,
      target: {value: '12345', selectionStart: 3, selectionEnd: 4},
      preventDefault: jest.genMockFn()
    };

    // We need to restrict this
    expect(component.restrictExpiry(event)).toBeTruthy();
  });

  it('ensures restrictExpiry passes with more than 6 digits', () => {
    // Dummy event
    const event = {
      keyCode: 53,
      which: 53,
      target: {value: '123456'},
      preventDefault: jest.genMockFn()
    };

    // We need to restrict this
    expect(component.restrictExpiry(event)).toBeFalsy();
  });
});
